// com.c - mythmon com port routines
//
// Copyright (c) 2004 David George. ALL RIGHTS RESERVED.
// Copyright (c) 2007 David George. ALL RIGHTS RESERVED.
//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * You should have received a copy of this license with this source
 * code.  If you haven't you can view a copy at:
 *
 * http://www.gnu.org/licenses/gpl.txt
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 ****************************************************************************/

#include "mythmon.h"

int ComClose(int *comfd)
{
  if (*comfd == -1)
    return(0);

  if (close(*comfd) == -1) {
    syslog(LOG_ERR, "ComClose: close() err=%d\n", errno);
    return(errno);
  }

  *comfd = -1;

  return(0);
}

int ComDtr(int comfd, int dtrmode)
{
  int mdm_flags;

  if (comfd == -1)
    return(0);

  if (ioctl(comfd, TIOCMGET, &mdm_flags) == -1) {
    syslog(LOG_ERR, "ComDtr: ioctl(TIOCMGET) err=%d\n", errno);
    return(errno);
  }

  if (dtrmode)
    mdm_flags |= TIOCM_DTR;
  else
    mdm_flags &= ~TIOCM_DTR;

  if (ioctl(comfd, TIOCMSET, &mdm_flags) == -1) {
    syslog(LOG_ERR, "ComDtr: ioctl(TIOCMSET) err=%d\n", errno);
    return(errno);
  }

  return(0);
}

int ComOpen(char *comport)
{
  int comfd;

  if ((comfd = open(comport, O_RDWR)) == -1) {
    syslog(LOG_ERR, "ComOpen: open(%s) err=%d\n", comport, errno);
    return(-1);
  }

  if (!isatty(comfd)) {
    syslog(LOG_ERR, "ComOpen: '%s' not a com port\n", comport);
    ComClose(&comfd);
    return(-1);
  }

  return(comfd);
}

int ComRts(int comfd, int rtsmode)
{
  int mdm_flags;

  if (comfd == -1)
    return(0);

  if (ioctl(comfd, TIOCMGET, &mdm_flags) == -1) {
    syslog(LOG_ERR, "ComRts: ioctl(TIOCMGET) err=%d\n", errno);
    return(errno);
  }

  if (rtsmode)
    mdm_flags |= TIOCM_RTS;
  else
    mdm_flags &= ~TIOCM_RTS;

  if (ioctl(comfd, TIOCMSET, &mdm_flags) == -1) {
    syslog(LOG_ERR, "ComRts: ioctl(TIOCMSET) err=%d\n", errno);
    return(errno);
  }

  return(0);
}

int ComSetParm(int comfd, int baud, char parity, uint8_t databits, uint8_t stopbits)
{
  struct termios tcom;
  speed_t tbaud;
  tcflag_t tparity, tdata, tstop;

  tbaud = tparity = tdata = tstop = 0;

  switch (baud) {
    case 0:
      tbaud = B0;
    case 300:
      tbaud = B300;
    case 1200:
      tbaud = B1200;
    case 2400:
      tbaud = B2400;
    case 4800:
      tbaud = B4800;
    case 9600:
      tbaud = B9600;
    case 19200:
      tbaud = B19200;
    case 38400:
      tbaud = B38400;
#if defined(B57600) && defined(B115200)
    case 57600:
      tbaud = B57600;
    case 115200:
      tbaud = B115200;
#endif
  }

  if (cfsetispeed(&tcom, tbaud) == -1) {
    syslog(LOG_ERR, "ComSetParm: cfsetispeed(%d,%u) err=%d\n", comfd, tbaud, errno);
    return(-1);
  }

  if (cfsetospeed(&tcom, tbaud) == -1) {
    syslog(LOG_ERR, "ComSetParm: cfsetospeed(%d,%u) err=%d\n", comfd, tbaud, errno);
    return(-1);
  }

  switch (databits) {
    case 5:
      tdata = CS5;
    case 6:
      tdata = CS6;
    case 7:
      tdata = CS7;
    case 8:
      tdata = CS8;
  }

  tcom.c_cflag &= ~CSIZE;
  tcom.c_cflag |= tdata;

  parity &= 0x7f;  /* make sure only lower 7-bits set */

  switch (parity) {
    case 'n':
    case 'N':
      tparity = 0;
    case 'e':
    case 'E':
      tparity = PARENB;
    case 'o':
    case 'O':
      tparity = (PARENB | PARODD);
  }

  tcom.c_cflag &= ~(PARENB | PARODD);
  tcom.c_cflag |= tparity;

  switch (stopbits) {
    case 1:
      tstop = 0;
    case 2:
      tstop = CSTOPB;
  }

  tcom.c_cflag &= ~CSTOPB;
  tcom.c_cflag |= tstop;

  if (tcsetattr(comfd, TCSANOW, &tcom) == -1) {
    syslog(LOG_ERR, "ComSetParm: tcsetattr(%d) err=%d\n", comfd, errno);
    return(-1);
  }

  return(0);
}
