// led.c - mythmon LED routines
//
// Copyright (c) 2004 David George. ALL RIGHTS RESERVED.
// Copyright (c) 2007 David George. ALL RIGHTS RESERVED.
//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * You should have received a copy of this license with this source
 * code.  If you haven't you can view a copy at:
 *
 * http://www.gnu.org/licenses/gpl.txt
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 ****************************************************************************/

#include "mythmon.h"

void do_cylon(int loops, int comfd, size_t ioport, int iotype)
{
  uint32_t led_state;
  int i, j;

  for (i = 0; i < loops; i++) {
    led_state = 1;
    for (j = 0; j < num_tuners; j++) {
      set_leds(led_state, comfd, ioport, iotype);
      led_state *= 4;
      my_sleep(100);
    }

    for (j = 0; j < num_tuners; j++) {
      set_leds(led_state, comfd, ioport, iotype);
      led_state /= 4;
      my_sleep(100);
    }
  }

  set_leds(0, comfd, ioport, iotype);

  return;
}

int set_leds(uint32_t led_state, int comfd, size_t ioport, int iotype)
{
  int i;

  if (comfd != -1) {
    ComRts(comfd, TRUE);
    for (i = 31; i >= 0; i--) {
      ComRts(comfd, (led_state >> i) & 0x01);
      ComDtr(comfd, TRUE);
      ComDtr(comfd, FALSE);
    }
    ComRts(comfd, FALSE);
  } else if (iotype == 2) {
    for (i = 31; i >= 0; i--) {
      outb(((led_state >> i) & 0x01), ioport);
      outb(((led_state >> i) & 0x01) | 0x02, ioport);
      outb(((led_state >> i) & 0x01), ioport);
    }
    outb(0x04, ioport);
    outb(0x00, ioport);
  } else if (iotype == 3) {
    int fd;

    if ((fd = open("/dev/mem", O_RDWR|O_SYNC)) != -1) {
      uint8_t *gpioregs;

      // map in DIO_0 to DIO_15
      gpioregs = (uint8_t *)mmap(0, getpagesize(), PROT_READ|PROT_WRITE,
                                 MAP_SHARED, fd, 0x12c00000);

      gpioregs[0] = 0xff;  // DDR set DIO_0 to DIO_7 to output
      gpioregs[1] = 0xff;  // DDR set DIO_8 to DIO_15 to output
      gpioregs[2] = led_state & 0xff;  // DR DIO_0 to DIO_7
      gpioregs[3] = (uint8_t)((led_state >> 8) & 0xff);  // DR DIO_8 to DIO_15

      munmap((void *)gpioregs, getpagesize());

      // map in DIO_16 to DIO_19
      gpioregs = (uint8_t *)mmap(0, getpagesize(), PROT_READ|PROT_WRITE,
                                 MAP_SHARED, fd, 0x12000000);

      // 0=TS-7400 control register
      gpioregs[1] = 0xf0;  // bit 7:4 DDR set DIO_16 to DIO_19 to output
      gpioregs[1] = (led_state >> 16) & 0x0f;  // bit 3:0 DR DIO_16 to DIO_19

      munmap((void *)gpioregs, getpagesize());

      close(fd);
    }
  } else if (iotype == 4 || iotype == 5) {
    uint8_t porta[1], portb[1], pinvals;
    int retval;

    porta[0] = portb[0] = 0;

    if (iotype == 5) {
      porta[0] = (uint8_t)(led_state & 0xff);
      portb[0] = (uint8_t)((led_state >> 8) & 0xff);
    } else if (iotype == 4) {
      if (led_state & 0x00000010)
        porta[0] |= 0x40;
      if (led_state & 0x00000020)
        porta[0] |= 0x80;
      if (led_state & 0x00000040)
        porta[0] |= 0x10;
      if (led_state & 0x00000080)
        porta[0] |= 0x20;
      if (led_state & 0x00000100)
        porta[0] |= 0x04;
      if (led_state & 0x00000200)
        porta[0] |= 0x08;
      if (led_state & 0x00000400)
        porta[0] |= 0x01;
      if (led_state & 0x00000800)
        porta[0] |= 0x02;

      if (led_state & 0x00000001)
        portb[0] |= 0x04;
      if (led_state & 0x00000002)
        portb[0] |= 0x08;
      if (led_state & 0x00000004)
        portb[0] |= 0x01;
      if (led_state & 0x00000008)
        portb[0] |= 0x02;
      if (led_state & LED_SPARE)
        portb[0] |= 0x10;
      if (led_state & LED_MESSAGE)
        portb[0] |= 0x20;
      if (led_state & LED_SYSTEM)
        portb[0] |= 0x40;
      if (led_state & CTRL_BELL)
        portb[0] |= 0x80;
    }

//    fprintf(stderr, "writing 0x%02x to porta\n", porta[0]);

    if ((retval = ftdi_write_data(&ftdic, porta, 1)) < 0) {
      syslog(LOG_ERR, "ftdi_write_data(porta,0x%02x) err=%d (%s)\n", porta[0], retval,
             ftdi_get_error_string(&ftdic));
      ftdi_disable_bitbang(&ftdic);
      ftdi_usb_close(&ftdic);
      ftdi_deinit(&ftdic);
      ftdi_disable_bitbang(&ftdic2);
      ftdi_usb_close(&ftdic2);
      ftdi_deinit(&ftdic2);
      return(-1);
    }

//    ftdi_read_pins(&ftdic, &pinvals);

//    fprintf(stderr, "porta pinstate=0x%02x\n", pinvals);

//    fprintf(stderr, "writing 0x%02x to portb\n", portb[0]);

    if ((retval = ftdi_write_data(&ftdic2, portb, 1)) < 0) {
      syslog(LOG_ERR, "ftdi_write_data(portb,0x%02x) err=%d (%s)\n", portb[0], retval,
             ftdi_get_error_string(&ftdic2));
      ftdi_disable_bitbang(&ftdic);
      ftdi_usb_close(&ftdic);
      ftdi_deinit(&ftdic);
      ftdi_disable_bitbang(&ftdic2);
      ftdi_usb_close(&ftdic2);
      ftdi_deinit(&ftdic2);
      return(-1);
    }

//    ftdi_read_pins(&ftdic2, &pinvals);

//    fprintf(stderr, "portb pinstate=0x%02x\n", pinvals);
  }

  return(0);
}
