// Myth monitor - monitor system and display status with LEDs
//
// Copyright (c) 2004 David George. ALL RIGHTS RESERVED.
// Copyright (c) 2007 David George. ALL RIGHTS RESERVED.
//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * You should have received a copy of this license with this source
 * code.  If you haven't you can view a copy at:
 *
 * http://www.gnu.org/licenses/gpl.txt
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 ****************************************************************************/

#define GLOBALDEFS
#include "mythmon.h"

int main(int argc, char *argv[])
{
  FILE *fp;
  pid_t pid;
  size_t ioport;
  uint8_t daemon, guidedata, voicemail, tuners[MAX_TUNERS], have_proto;
  uint8_t led_arr[32], message_sent;
  uint32_t freespace, led_state, last_led_state, amount_free;
  uint32_t nread, nwritten;
  char buff[1025], command[81], host[81], comport[33], myhostname[65];
  char notify_string[257];
  char *ptr;
  int c, i, retval, comfd, netfd, commandlen, port, chars;
  int counter, guide_avail, interval, protover, iotype;
  int backend_proto;

  daemon = guidedata = voicemail = FALSE;
  message_sent = FALSE;
  have_sigterm = have_sighup = FALSE;
  freespace = 0;
  led_state = 0;
  num_tuners = MAX_TUNERS;
  interval = CHECK_INTERVAL;
  port = 6543;
  comfd = netfd = -1;
  ioport = 0;
  iotype = 0;
  protover = backend_proto = 0;
  memset(comport, 0, sizeof(comport));
  memset(tuners, 0, sizeof(tuners));
  memset(led_arr, 0, sizeof(led_arr));
  memset(notify_string, 0, sizeof(notify_string));

  gethostname(myhostname, sizeof(myhostname)-1);

  while (TRUE) {
    int this_option_optind = optind ? optind : 1;
    int option_index = 0;
    static struct option long_options[] = {
      {"freespace", 1, 0, 0},
      {"daemon", 0, 0, 0},
      {"comport", 1, 0, 0},
      {"guidedata", 0, 0, 0},
      {"host", 1, 0, 0},
      {"interval", 1, 0, 0},
      {"ioport", 1, 0, 0},
      {"iotype", 1, 0, 0},
      {"notify", 1, 0, 0},
      {"port", 1, 0, 0},
      {"protover", 1, 0, 0},
      {"tuners", 1, 0, 0},
      {"version", 0, 0, 0},
      {"voicemail", 0, 0, 0},
      {0, 0, 0, 0}
    };

    c = getopt_long(argc, argv, "dh:i:n:p:t:v", long_options, &option_index);
    if (c == -1)
      break;

    switch (c) {
      case 0:
        /*
        printf("option %s", long_options[option_index].name);
        if (optarg)
          printf(" with arg %s", optarg);
        printf("\n");
        */
        if (option_index == 0 && optarg)
          freespace = atoi(optarg);
        else if (option_index == 1)
          daemon = TRUE;
        else if (option_index == 2 && optarg)
          strnzcpy(comport, optarg, sizeof(comport));
        else if (option_index == 3)
          guidedata = TRUE;
        else if (option_index == 4 && optarg)
          strnzcpy(host, optarg, sizeof(host));
        else if (option_index == 5 && optarg)
          interval = atoi(optarg);
        else if (option_index == 6 && optarg) {
          if (!strncmp2(optarg, "0x"))
            ptr = optarg + 2;
          else
            ptr = optarg;

          ioport = htoi(ptr);
        } else if (option_index == 7 && optarg)
          iotype = atoi(optarg);
        else if (option_index == 8 && optarg)
          snprintf(notify_string, sizeof(notify_string)-1,
                   "echo 'Check MythTV backend' | mail -s mythtv %s", optarg);
        else if (option_index == 9 && optarg)
          port = atoi(optarg);
        else if (option_index == 10 && optarg) {
          if (!strncmp(optarg, "auto", 4))
            protover = 0;
          else if (isdigit(*optarg))
            protover = atoi(optarg);
        } else if (option_index == 11 && optarg)
          num_tuners = atoi(optarg);
        else if (option_index == 12) {
          printf("\nmythmon version %s\n\n", MYTHMON_VERSION);
          exit(0);
        } else if (option_index == 13)
          voicemail = TRUE;
        break;

      case 'd':
        daemon = TRUE;
        break;

      case 'h':
        strnzcpy(host, optarg, sizeof(host));
        break;

      case 'i':
        interval = atoi(optarg);
        break;

      case 'n':
        snprintf(notify_string, sizeof(notify_string)-1,
                 "echo 'Check MythTV backend' | mail -s mythtv %s", optarg);
        break;

      case 'p':
        port = atoi(optarg);
        break;

      case 't':
        num_tuners = atoi(optarg);
        break;

      case 'v':
        printf("\nmythmon version %s\n\n", MYTHMON_VERSION);
        exit(0);

      default:
        printf("\nmythmon version %s\n\n", MYTHMON_VERSION);
        puts("Usage: mythmon [args]");
        puts("       --freespace amount (monitor free disk space in MB)");
        puts("       -d | --daemon (run as a background daemon)");
        puts("       --comport comport (send LED info using comport)");
        puts("       --guidedata (monitor available guidedata)");
        puts("       -h | --host hostname (backend host name)");
        puts("       -i | --interval (backend query interval in seconds)");
        puts("       --ioport (send LED info using ioport in hex)");
        puts("       --iotype (0=auto, 1=COM, 2=LV667, 3=TS7400, 4=2232D old, 5=2232D)");
        puts("       -n | --notify emailaddr (notify email address on backend failure)");
        puts("       -p | --port port (backend port number)");
        puts("       --protover version (0=auto, version of MythTV backend)");
        puts("       -t | --tuners max (number of tuners to check)");
        puts("       -v | --version (display version)");
        puts("       --voicemail (enable message waiting indicator)");
        puts("");
        exit(0);
    }
  }

  openlog("mythmon", LOG_PID, SYSLOG_FACILITY);

  if (iotype == 1 && !(*comport)) {
    if (!daemon)
      fprintf(stderr, "iotype set to COM, but comport not specified\n");
    syslog(LOG_ERR, "iotype set to COM, but comport not specified\n");
    exit(1);
  }

  if (iotype == 0) {
    if (ioport == 0x0208)
      iotype = 1;  // assume Commell LV667
    else if (ioport == 0x12c00000)
      iotype = 2;  // assume TS7400
    else if (ioport == 0)
      iotype = 5;  // assume FTDI 2232D new style
  }

  if ((fp = fopen("/var/run/mythmon.pid", "r")) != NULL) {
    fgets(buff, sizeof(buff)-1, fp);
    fclose(fp);

    pid = (pid_t)atoi(buff);
    sprintf(buff, "/proc/%d", pid);
    if (access(buff, 0) == 0) {
      if (!daemon)
        fprintf(stderr, "A mythmon process is already running\n");
      syslog(LOG_ERR, "A mythmon process is already running\n");
      exit(0);
    }
  }

  // detach ourself from the controlling terminal
  if (daemon) {
    if ((pid = fork()) < 0)
      return(-1);
    else if (pid != 0)
      exit(0);  // parent goes bye-bye

    setsid();
    umask(0022);
  }

  if ((fp = fopen("/var/run/mythmon.pid", "w")) != NULL) {
    fprintf(fp, "%d\n", getpid());
    fclose(fp);
  }

  {
    struct sigaction sigact;
    sigset_t blockset;

    // signals to ignore
    sigemptyset(&blockset);
    sigact.sa_mask = blockset;
    sigact.sa_handler = SIG_IGN;
    if (daemon) {
      if (sigaction(SIGINT, &sigact, NULL) == -1)
        syslog(LOG_WARNING, "Couldn't register signal handler for SIGINT.\n");
    }
    // so we don't have to do a wait to clean up our zombies
    if (sigaction(SIGCHLD, &sigact, NULL) == -1)
      syslog(LOG_WARNING, "Couldn't register signal handler for SIGCHLD.\n");

    // signals to catch
    memset(&sigact, 0, sizeof(struct sigaction));
    sigemptyset(&blockset);
    sigact.sa_mask = blockset;
    sigact.sa_flags = SA_SIGINFO;
    sigact.sa_sigaction = &sighandler;
    if (sigaction(SIGHUP, &sigact, NULL) == -1)
      syslog(LOG_WARNING, "Couldn't register signal handler for SIGHUP.\n");
    if (sigaction(SIGTERM, &sigact, NULL) == -1)
      syslog(LOG_WARNING, "Couldn't register signal handler for SIGTERM.\n");
    if (!daemon) {
      if (sigaction(SIGINT, &sigact, NULL) == -1)
        syslog(LOG_WARNING, "Couldn't register signal handler for SIGINT.\n");
    }

    sigaddset(&blockset, SIGHUP);
    sigaddset(&blockset, SIGTERM);
    sigaddset(&blockset, SIGINT);
    sigaddset(&blockset, SIGCHLD);
  }

  openlog("mythmon", LOG_PID, SYSLOG_FACILITY);

  if ((iotype == 0 || iotype == 1) && *comport) {
    syslog(LOG_DEBUG, "initializing comport %s\n", comport);

    comfd = ComOpen(comport);
    ComDtr(comfd, FALSE);
    ComRts(comfd, FALSE);
    ComSetParm(comfd, 9600, 'n', 8, 1);
  } else if (iotype == 2) {
    init_lv667(ioport);
  } else if (iotype == 4 || iotype == 5) {
    if (init_ftdi())
      exit(-7);
  } else if (iotype != 3) {
    fprintf(stderr, "Must specify either a comport or an ioport\n");
    exit(-6);
  }

  // little power-up display ;-)
  do_cylon(2, comfd, ioport, iotype);

lost_backend:
  counter = 0;
  while (netfd == -1) {
    if ((counter % 5) == 0) {
      syslog(LOG_DEBUG, "opening connection to %s:%d\n", host, port);

      // make sure light and alarm off while attempting
      led_state = 0;
      set_leds(led_state, comfd, ioport, iotype);

      netfd = NetClientOpen(host, port);

      counter = 0;
    }

    if (led_state == 0 && netfd == -1)
      led_state = ALERT1;
    else
      led_state = 0;

    set_leds(led_state, comfd, ioport, iotype);

    if (netfd == -1) {
      if (*notify_string && message_sent == FALSE) {
        system(notify_string);
        message_sent = TRUE;
      }

      my_sleep(1000L);
      ++counter;
    }

    if (have_sighup || have_sigterm)
      goto shutdown;
  }

  message_sent = FALSE;

  // check backend version
  have_proto = FALSE;
//  do {
    snprintf(command, sizeof(command)-1, "MYTH_PROTO_VERSION %d", protover);
    snprintf(buff, sizeof(buff)-1, "%-8d%s", strlen(command), command);
    retval = NetWriteString(netfd, buff);
    if (retval) {
      NetClose(&netfd);
      goto lost_backend;
    }

    retval = NetReadTime(netfd, buff, sizeof(buff)-1, 30000, 1000, &nread);
    if (retval) {
      NetClose(&netfd);
      goto lost_backend;
    }
    buff[nread] = '\0';

    if (!daemon)
      fprintf(stderr, "backend returned version=%d '%s'\n", strlen(buff), buff);

    if (strstr(buff, "ACCEPT") != NULL) {
      have_proto = TRUE;
    } else {
      char *tmpptr = strstr(buff, "[]:[]");

      if (tmpptr != NULL) {
        backend_proto = atoi(tmpptr+5);

        if (!daemon)
          fprintf(stderr, "Myth protocol version mismatch(us=%d,backend=%d)",
                  protover, backend_proto);

        syslog(LOG_ERR, "Myth protocol version mismatch(us=%d,backend=%d)",
               protover, backend_proto);

        if (protover == 0)
          protover = backend_proto;
        else
          goto shutdown;
      }
    }

    if (have_sigterm || have_sighup)
      goto shutdown;

//    if (!have_proto)
//      sleep(10);
//  } while (!have_proto);
/*
  if (backend_proto == 0) {
    NetClose(&netfd);
    goto lost_backend;
  }
*/
  snprintf(command, sizeof(command)-1, "ANN Playback %s 0", myhostname);
  snprintf(buff, sizeof(buff)-1, "%-8d%s", strlen(command), command);
  retval = NetWriteString(netfd, buff);
  if (retval) {
    NetClose(&netfd);
    goto lost_backend;
  }
  retval = NetReadTime(netfd, buff, 8, 30000, 1000, &nread);
  if (retval) {
    NetClose(&netfd);
    goto lost_backend;
  }
  commandlen = atoi(buff);
  retval = NetReadTime(netfd, buff, commandlen, 30000, 1000, &nread);
  if (retval) {
    NetClose(&netfd);
    goto lost_backend;
  }
  buff[nread] = '\0';

  counter = 0;
  last_led_state = led_state;
  while (TRUE) {
    // we need to loop anyway in order to support flashing LEDs so only
    // send requests every interval seconds
    if ((counter % interval) == 0) {
      if (freespace) {
        // if low set system led to flash
        strnscpy(command, "QUERY_FREESPACE", sizeof(command));
        chars = snprintf(buff, sizeof(buff)-1, "%-8d%s", strlen(command),
                         command);
        retval = NetWrite(netfd, buff, chars, &nwritten);
        if (retval || !nwritten) {
          NetClose(&netfd);
          goto lost_backend;
        }
        retval = NetReadTime(netfd, buff, 8, 30000, 1000, &nread);
        if (retval || !nread) {
          NetClose(&netfd);
          goto lost_backend;
        }
        buff[nread] = '\0';
        commandlen = atoi(buff);
        retval = NetReadTime(netfd, buff, commandlen, 30000, 1000, &nread);
        if (retval || !nread) {
          NetClose(&netfd);
          goto lost_backend;
        }
        buff[nread] = '\0';

        if (!daemon)
          fprintf(stderr, "freespace='%s'\n", buff);

        amount_free = 0;
        if ((ptr = strstr(buff, "[]:[]")) != NULL) {
          ptr += 5;
          amount_free = (uint32_t)atol(buff) - (uint32_t)atol(ptr);
        }

        if (amount_free < freespace)
          fprintf(stderr, "Warning, low disk space\n");

        if (led_arr[IDX_SYSTEM] < LED_FLASH && amount_free < freespace)
          led_arr[IDX_SYSTEM] = LED_FLASH;
      }

      if (guidedata) {
        // if less than 7 days set system led
        // if less than 3 days set system led to flash
        strnscpy(command, "QUERY_GUIDEDATATHROUGH", sizeof(command));
        snprintf(buff, sizeof(buff)-1, "%-8d%s", strlen(command), command);
        retval = NetWriteString(netfd, buff);
        if (retval) {
          NetClose(&netfd);
          goto lost_backend;
        }
        retval = NetRead(netfd, buff, 8, &nread);
        if (retval) {
          NetClose(&netfd);
          goto lost_backend;
        }
        commandlen = atoi(buff);
        retval = NetRead(netfd, buff, commandlen, &nread);
        if (retval) {
          NetClose(&netfd);
          goto lost_backend;
        }
        buff[nread] = '\0';

        if (!daemon)
          fprintf(stderr, "guide='%s'\n", buff);

        guide_avail = atoi(buff);
        if (led_arr[IDX_SYSTEM] < LED_ON && guide_avail < 7)
          led_arr[IDX_SYSTEM] = LED_ON;
        else if (led_arr[IDX_SYSTEM] < LED_FLASH && guide_avail < 3)
          led_arr[IDX_SYSTEM] = LED_FLASH;
      }

      if (voicemail) {
        strnscpy(command, "QUERY_NEWMESSAGES", sizeof(command));
        snprintf(buff, sizeof(buff)-1, "%-8d%s", strlen(command), command);
        retval = NetWriteString(netfd, buff);
        if (retval) {
          NetClose(&netfd);
          goto lost_backend;
        }
        retval = NetRead(netfd, buff, 8, &nread);
        if (retval) {
          NetClose(&netfd);
          goto lost_backend;
        }
        commandlen = atoi(buff);
        retval = NetRead(netfd, buff, commandlen, &nread);
        if (retval) {
          NetClose(&netfd);
          goto lost_backend;
        }
        buff[nread] = '\0';

        if (!daemon)
          fprintf(stderr, "voicemail='%s'\n", buff);

        if (atoi(buff))
          led_arr[IDX_MESSAGE] = LED_ON;
      }

      for (i = 1; i <= num_tuners; i++) {
        // set tuner led based on tuners status
        snprintf(command, sizeof(command)-1,
                 "QUERY_RECORDER %d []:[]IS_RECORDING", i);
        chars = snprintf(buff, sizeof(buff)-1, "%-8d%s", strlen(command),
                         command);
        retval = NetWrite(netfd, buff, chars, &nwritten);
        if (retval || !nwritten) {
          NetClose(&netfd);
          goto lost_backend;
        }
        retval = NetReadTime(netfd, buff, 8, 3000, 1000, &nread);
        if (retval || !nread) {
          NetClose(&netfd);
          goto lost_backend;
        }
        commandlen = atoi(buff);
        retval = NetReadTime(netfd, buff, commandlen, 30000, 1000, &nread);
        if (retval || !nread) {
          NetClose(&netfd);
          goto lost_backend;
        }
        buff[nread] = '\0';

        if (!daemon)
          fprintf(stderr, "tuner %d='%s'\n", i, buff);

        if (!strncmp2(buff, "bad"))
          led_arr[i-1] = LED_OFF;
        else if (isdigit(buff[0]) && atoi(buff) == 0)
          led_arr[i-1] = LED_GREEN;
        else if (isdigit(buff[0]) && atoi(buff) == 1)
          led_arr[i-1] = LED_RED;
      }

      counter = 0;
    }

    led_state &= ~(0x5fff);
    for (i = 0; i < num_tuners; i++) {
      if (led_arr[i] == LED_RED)
        led_state |= (1 << (i * 2));
      else if (led_arr[i] == LED_GREEN)
        led_state |= (2 << (i * 2));
    }

    if (led_arr[IDX_SYSTEM] == LED_OFF)
      led_state &= ~(LED_SYSTEM);
    else if (led_arr[IDX_SYSTEM] == LED_ON)
      led_state |= LED_SYSTEM;
    else if (led_arr[IDX_SYSTEM] == LED_FLASH) {
      if (led_state & LED_SYSTEM)
        led_state &= ~(LED_SYSTEM);
      else
        led_state |= LED_SYSTEM;
    }

    if (have_sigterm || have_sighup)
      break;

    retval = set_leds(led_state, comfd, ioport, iotype);
    if (retval) {
      NetClose(&netfd);
      goto lost_backend;
    }

    my_sleep(1000L);
    ++counter;
  }

shutdown:
  set_leds(0, comfd, ioport, iotype);

  if (*comport && comfd != -1) {
    syslog(LOG_DEBUG, "shutting down %s\n", comport);
    ComDtr(comfd, FALSE);
    ComRts(comfd, FALSE);
    ComClose(&comfd);
  } else if (iotype == 2) {
    outb(0x00, ioport);
  } else if (iotype == 4 || iotype == 5) {
    ftdi_disable_bitbang(&ftdic);
    ftdi_usb_close(&ftdic);
    ftdi_deinit(&ftdic);
    ftdi_disable_bitbang(&ftdic2);
    ftdi_usb_close(&ftdic2);
    ftdi_deinit(&ftdic2);
  }

  NetClose(&netfd);
  exit(0);
}
