// misc.c - mythmon miscellaneous routines
//
// Copyright (c) 2004 David George. ALL RIGHTS RESERVED.
// Copyright (c) 2007 David George. ALL RIGHTS RESERVED.
//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * You should have received a copy of this license with this source
 * code.  If you haven't you can view a copy at:
 *
 * http://www.gnu.org/licenses/gpl.txt
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 ****************************************************************************/

#include "mythmon.h"

int init_ftdi()
{
  uint16_t vendorid, productid;
  int retval;

  vendorid = 0x0403;
  productid = 0x6010;

  if ((retval = ftdi_init(&ftdic)) < 0) {
    syslog(LOG_ERR, "ftdi_init(porta) err=%d (%s)\n", retval, ftdi_get_error_string(&ftdic));
    return(-1);
  }

  if ((retval = ftdi_set_interface(&ftdic, INTERFACE_A)) < 0) {
    syslog(LOG_ERR, "ftdi_set_interface(porta) err=%d (%s)\n", retval,
           ftdi_get_error_string(&ftdic));
    ftdi_deinit(&ftdic);
    return(-1);
  }

  if ((retval = ftdi_usb_open(&ftdic, vendorid, productid)) < 0) {
    if (retval != -5)
      syslog(LOG_ERR, "ftdi_usb_open(porta,vend=0x%04x,prod=0x%04x) err=%d (%s)\n",
             vendorid, productid, retval, ftdi_get_error_string(&ftdic));
    ftdi_deinit(&ftdic);
    return(-1);
  }

  if ((retval = ftdi_set_bitmode(&ftdic, 0xff, 1)) < 0) {
    syslog(LOG_ERR, "ftdi_enable_bitbang(porta) err=%d (%s)\n", retval,
           ftdi_get_error_string(&ftdic));
    ftdi_usb_close(&ftdic);
    ftdi_deinit(&ftdic);
    return(-1);
  }

  if ((retval = ftdi_init(&ftdic2)) < 0) {
    syslog(LOG_ERR, "ftdi_init(portb) err=%d (%s)\n", retval, ftdi_get_error_string(&ftdic2));
    ftdi_disable_bitbang(&ftdic);
    ftdi_usb_close(&ftdic);
    ftdi_deinit(&ftdic);
    return(-1);
  }
  if ((retval = ftdi_set_interface(&ftdic2, INTERFACE_B)) < 0) {
    syslog(LOG_ERR, "ftdi_set_interface(portb) err=%d (%s)\n", retval,
           ftdi_get_error_string(&ftdic2));
    ftdi_disable_bitbang(&ftdic);
    ftdi_usb_close(&ftdic);
    ftdi_deinit(&ftdic);
    ftdi_deinit(&ftdic2);
    return(-1);
  }
  if ((retval = ftdi_usb_open(&ftdic2, vendorid, productid)) < 0) {
    if (retval != -5)
      syslog(LOG_ERR, "ftdi_usb_open(portb,vend=0x%04x,prod=0x%04x) err=%d (%s)\n",
             vendorid, productid, retval, ftdi_get_error_string(&ftdic2));
    ftdi_disable_bitbang(&ftdic);
    ftdi_usb_close(&ftdic);
    ftdi_deinit(&ftdic);
    ftdi_deinit(&ftdic2);
    return(-1);
  }

  if ((retval = ftdi_set_bitmode(&ftdic2, 0xff, 1)) < 0) {
    syslog(LOG_ERR, "ftdi_enable_bitbang(portb) err=%d (%s)\n", retval,
           ftdi_get_error_string(&ftdic2));
    ftdi_disable_bitbang(&ftdic);
    ftdi_usb_close(&ftdic);
    ftdi_deinit(&ftdic);
    ftdi_usb_close(&ftdic2);
    ftdi_deinit(&ftdic2);
    return(-1);
  }

  return(0);
}

void init_lv667(size_t ioport)
{
  syslog(LOG_DEBUG, "initializing LV667 ioport 0x%x\n", ioport);

  if (ioperm(1, ioport, 1) == 0) {
    // setup W83697UF for programming
    outb(0x87, 0x2e);
    outb(0x87, 0x2e);
    // set logical port to 7
    outb(0x07, 0x2e);
    outb(0x07, 0x2f);
    // CR30 set logical device 7 active
    outb(0x30, 0x2e);
    outb(0x01, 0x2f);
    // set all pins to output
    outb(0xf0, 0x2e);
    outb(0x0, 0x2f);  // 0xf8 for 3 bit
    // CR29 change Game port to GPIO
    outb(0x29, 0x2e);
    outb(0x80, 0x2f);
    // set base address to 0x208
    outb(0x62, 0x2e);
    outb(ioport>>8, 0x2f);
    outb(0x63, 0x2e);
    outb(ioport&0xff, 0x2f);
    // done programming W83697UF
    outb(0xaa, 0x2e);

    outb(0, ioport);
  }

  return;
}

int my_sleep(int ms)
{
  struct timespec ts;

  if (ms > 0) {
    ts.tv_sec = ms / 1000;
    ts.tv_nsec = (ms % 1000) * 1000000;

    return(nanosleep(&ts, NULL));
  }

  return(0);
}

int strncmp2(const char *str1, const char *str2)
{
  return(strncmp(str1, str2, strlen(str2)));
}

int strnicmp2(const char *str1, const char *str2)
{
  return(strnicmp(str1, str2, strlen(str2)));
}

char *strnscpy(char *dest, const char *src, const int bytes)
{
  dest[bytes] = '\0';
  return(strncpy(dest, src, bytes-1));
}

char *strnzcpy(char *dest, const char *src, const int bytes)
{
  memset(dest, 0, bytes);
  return(strncpy(dest, src, bytes-1));
}

uint32_t htoi(char *str)
{
  char *ptr = str;
  uint32_t hexval = 0;

  if (strlen(ptr) > 8)
    return(0);

  while (*ptr) {
    hexval <<= 4;
    if (*ptr >= '0' && *ptr <= '9')
      hexval += *ptr - '0';
    else if (*ptr >= 'a' && *ptr <= 'f')
      hexval += *ptr - 'a' + 10;
    else if (*ptr >= 'A' && *ptr <= 'F')
      hexval += *ptr - 'A' + 10;
    else
      break;  // stop at first non-hex char

    ++ptr;
  }

  return(hexval);
}

void sighandler(int signum, siginfo_t *info, void *extra)
{
  if (signum == SIGTERM || signum == SIGINT)
    have_sigterm = TRUE;
  else if (signum == SIGHUP)
    have_sighup = TRUE;
}

int is_num_address(char *addr)
{
  char *ptr;
  int retval;

  ptr = addr;
  retval = TRUE;
  while (*ptr) {
    if (!isdigit(*ptr) && *ptr != '.') {
      retval = FALSE;
      break;
    }
    ++ptr;
  }

  return(retval);
}
