// Myth monitor - monitor system and display status with LEDs
//
// Copyright (c) 2004 David George. ALL RIGHTS RESERVED.
// Copyright (c) 2007 David George. ALL RIGHTS RESERVED.
//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * You should have received a copy of this license with this source
 * code.  If you haven't you can view a copy at:
 *
 * http://www.gnu.org/licenses/gpl.txt
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 ****************************************************************************/

#ifndef _MYTHMON_H
#define _MYTHMON_H

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>     // memset, str*
#include <errno.h>

#include <getopt.h>
#include <syslog.h>
#include <ctype.h>      // isdigit, ...

#include <termios.h>    // struct termios
#include <termio.h>     // TIOCMGET, TIOCMSET
#include <fcntl.h>      // O_RDWR
#include <sys/io.h>     // ioperm, outb
#include <signal.h>     // SIGINT, SIGTERM, SIG_IGN, etc
#include <sys/mman.h>   // mmap

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>      // gethost...()

#include <strings.h>        // strcasecmp(), strncasecmp()
#define stricmp(x, y)       strcasecmp((x), (y))
#define strnicmp(x, y, z)   strncasecmp((x), (y), (z))

// FTDI 2232D - http://ftdichip.com
#include <ftdi.h>

#define MYTH_PROTO_VERSION 43  // svn 19465
#define MYTHMON_VERSION    "3.0"
#define MAX_TUNERS         8     // number of actual tuner LEDs you have (1-8)
#define CHECK_INTERVAL     20    // send backend queries every 20 seconds
#define SYSLOG_FACILITY    LOG_LOCAL6

#define LED_OFF 0
#define LED_ON 1
#define LED_RED 1
#define LED_GREEN 2
#define LED_FLASH 3

// each tuner needs two bits (one for green and one for red)
#define LED_TUNER1       0x00000001
#define LED_TUNER2       0x00000004
#define LED_TUNER3       0x00000010
#define LED_TUNER4       0x00000040
#define LED_TUNER5       0x00000100
#define LED_TUNER6       0x00000400
#define LED_TUNER7       0x00001000
#define LED_TUNER8       0x00004000

// start these right after the tuners so we can use a 20 bit A6812 instead of
// the 32 bit A6818
// also needed for 20-bit DIO on TS-7400
#define LED_SYSTEM       0x00010000
#define LED_MESSAGE      0x00020000
#define LED_SPARE        0x00040000
#define CTRL_BELL        0x00080000

#define ALERT1 (LED_TUNER1|LED_TUNER2|LED_TUNER3|LED_TUNER4|LED_TUNER5|LED_TUNER6|LED_TUNER7|LED_TUNER8|LED_SYSTEM|LED_MESSAGE|CTRL_BELL)
#define ALERT2 (((LED_TUNER1|LED_TUNER2|LED_TUNER3|LED_TUNER4|LED_TUNER5|LED_TUNER6|LED_TUNER7|LED_TUNER8)<<1)|LED_SYSTEM|LED_MESSAGE|CTRL_BELL)

#define IDX_SYSTEM       8
#define IDX_MESSAGE      9
#define IDX_BELL         10

#ifndef TRUE
#define TRUE (1)
#endif
#ifndef FALSE
#define FALSE (0)
#endif

#ifdef GLOBALDEFS
struct ftdi_context ftdic, ftdic2;
uint8_t have_sigterm, have_sighup;
int num_tuners;
#else
extern struct ftdi_context ftdic, ftdic2;
extern uint8_t have_sigterm, have_sighup;
extern int num_tuners;
#endif

// com.c
int ComClose(int *comfd);
int ComDtr(int comfd, int dtrmode);
int ComOpen(char *comport);
int ComRts(int comfd, int rtsmode);
int ComSetParm(int comfd, int baud, char parity, uint8_t databits, uint8_t stopbits);
// led.c
void do_cylon(int loops, int comfd, size_t ioport, int iotype);
int set_leds(uint32_t led_state, int comfd, size_t ioport, int iotype);
// misc.c
int init_ftdi();
void init_lv667(size_t ioport);
int my_sleep(int ms);
int strncmp2(const char *str1, const char *str2);
int strnicmp2(const char *str1, const char *str2);
char *strnscpy(char *dest, const char *src, const int bytes);
char *strnzcpy(char *dest, const char *src, const int bytes);
uint32_t htoi(char *str);
void sighandler(int signum, siginfo_t *info, void *extra);
int is_num_address(char *addr);
// net.c
int NetClientOpen(char *host, int port);
int NetClose(int *netfd);
int NetRead(int netfd, const void *vptr, size_t n, uint32_t *ulRead);
int NetReadTime(int netfd, char *str, int n, int init_time,
                int sec_time, uint32_t *ulRead);
int NetWrite(int netfd, const void *vptr, size_t n, uint32_t *ulWritten);
int NetWriteString(int netfd, const void *vptr);

#endif
